#ifndef __PENGUIN_GENERAL_HEURISTIC_HPP__
#define __PENGUIN_GENERAL_HEURISTIC_HPP__

#include "penguin.hpp"
#include "heuristic.hpp"
#include "penguin_heuristic.hpp"
#include <algorithm>

namespace mcts
{
	class generalization_heuristic : public penguin_heuristic
	{
		public:
	    float get_value(const game::penguin& game, uint8_t move) const 
	    {
			float result = 0;
			float B_free_move=0.15, B_num_dir=0.05, B_points=0.05, B_zone=0.15;
			float A_free_move=1.0, A_num_dir=1.0, A_points=0.5, A_zone=1.5;
			//uint64_t obstacles = game.create_obstacles_bitboard();
			game::penguin g = *(game::copy(game));
			g.play(move);
			auto actual_state = g.get_state();

			uint64_t obstacles = (~(actual_state.one_fish | actual_state.two_fish | actual_state.three_fish));
			for(int i = 0; i < 4; i++){
				obstacles |= ((uint64_t) 1) << PENGUIN_POS(actual_state.peng_red[i]);
				obstacles |= ((uint64_t) 1) << PENGUIN_POS(actual_state.peng_blue[i]);
			}

			int nbr_turns = 0;  // It count also the penguins so in this implementation it start at 10
			for (int i =0; i<60; i++){
				if((obstacles >> i) & 1) nbr_turns++;
			}

			result+= ((A_free_move/nbr_turns) + B_free_move) * movement_freedom_heuristic().get_value(game, move)
				+ ((A_num_dir/nbr_turns) + B_num_dir) * number_direction_freedom_heuristic().get_value(game, move)
				+ ((A_points/nbr_turns) + B_points) * points_heuristic().get_value(game, move)
				+ ((A_zone/nbr_turns) + B_zone) * zone_heuristic().get_value(game, move);
			//std::cout<< nbr_turns<< "  :  "<< result << std::endl;
			
			return result;
			
			
		}
	    int get_count(const game::penguin& game, uint8_t move) const
	    {

			return 100;
		}
	
    };
};

#endif
